# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class Atellica800AstmSegmentH(HeaderSegment):
    instrument_name = HprimString((5, 1))
    instrument_type = HprimString((5, 2))
    sw_version = HprimString((5, 3))
    sw_id = HprimString((5, 4))
    serial_number = HprimString((5, 5))
    test_type = HprimString(12, default="P")
    protocol = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Atellica800AstmSegmentP(Segment):
    patient_name = HprimString(6)

    orders = HprimForeign("O", "patient")


class Atellica800AstmSegmentO(Segment):
    sample_id = HprimString((3, 1))
    lot_number = HprimString((3, 4))
    expiration_date = HprimDate((3, 5))
    level = HprimString((3, 6))
    rack = HprimString((4, 1))
    position = HprimString((4, 2))
    dilution_factor = HprimString((5, 4))
    color_clarity = HprimString((5, 5))
    reagent_lot_number = HprimString((5, 6))
    reagent_exporation_date = HprimDate((5, 7))
    cassette_serial_number = HprimString((5, 8))
    priority = HprimString(6)
    request_date = HprimDate(7)
    action_code = HprimString(12)
    result_date = HprimDate(15)
    specimen = HprimString(16)
    report_type = HprimString(26, default="Q")

    results = HprimForeign("R", "order")


class Atellica800AstmSegmentQ(Segment):
    sample_id = HprimString((3, 2))
    query_status = HprimString(13)


class Atellica800AstmSegmentR(Segment):
    code_test = HprimString((3, 4))
    test_name = HprimString((3, 6))
    result = HprimString(4)
    unit = HprimString(5)
    abnormal_flags = HprimString(7)
    operator = HprimString(11)
    analyzer = HprimString(14)


class Atellica800AstmSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class Atellica800AstmSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "atellica800_cyto_astm",
        "label": "Atellica AU800 ASTM Cytologie Urinaire",
    }
    SEGMENT_CLASS_MAP = {
        "H": Atellica800AstmSegmentH,
        "P": Atellica800AstmSegmentP,
        "O": Atellica800AstmSegmentO,
        "Q": Atellica800AstmSegmentQ,
        "R": Atellica800AstmSegmentR,
        "C": Atellica800AstmSegmentC,
        "L": Atellica800AstmSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "atellica_cyto_cristaux",
            label="Liste des cristaux séparés par des \";\"",
            default="CRY;.CaOx;.CaOxm;.CaOxd;.TRI;.URI;.CapH;.U-AMO;.P-AMO;.CYS;.LEU;.TYR;.ATY"
        ),
        AstmMessageV2.Option(
            "atellica_cyto_cylindres",
            label="Liste des cylindres séparés par des \";\"",
            default="HYA;PAT;.C-HGR;.C-GRA;.C-NEC;.C-RBC;.C-WBC;.C-CRY;.C-MIC;.C-FAT;.C-WAX;.C-MIX"
        ),
        AstmMessageV2.BooleanOption(
            "atellica_cyto_ignorer_resultats_na",
            label="Ignorer les résultats des cristaux et des cylindres de valeur N/A",
            default=False
        ),
        AstmMessageV2.Option(
            "atellica_cyto_exclusion_resultat_cristaux",
            label="Liste des résultats des cristaux séparés par des \";\" à ignorer",
            default=""
        ),
        AstmMessageV2.Option(
            "atellica_cyto_exclusion_resultat_cylindres",
            label="Liste des résultats des cylindres séparés par des \";\" à ignorer",
            default=""
        ),
    ]

    def __init__(self, **kwargs):
        self.atellica_cyto_cristaux = kwargs.pop(
            "atellica_cyto_cristaux",
            "CRY;.CaOx;.CaOxm;.CaOxd;.TRI;.URI;.CapH;.U-AMO;.P-AMO;.CYS;.LEU;.TYR;.ATY"
        )
        self.atellica_cyto_cylindres = kwargs.pop(
            "atellica_cyto_cylindres",
            "HYA;PAT;.C-HGR;.C-GRA;.C-NEC;.C-RBC;.C-WBC;.C-CRY;.C-MIC;.C-FAT;.C-WAX;.C-MIX"
        )
        self.atellica_cyto_ignorer_resultats_na = kwargs.pop("atellica_cyto_ignorer_resultats_na", False)
        atellica_cyto_exclusion_resultat_cristaux = kwargs.pop("atellica_cyto_exclusion_resultat_cristaux", False)
        if atellica_cyto_exclusion_resultat_cristaux:
            self.liste_atellica_cyto_exclusion_resultat_cristaux = atellica_cyto_exclusion_resultat_cristaux.split(";")
        else:
            self.liste_atellica_cyto_exclusion_resultat_cristaux = []
        atellica_cyto_exclusion_resultat_cylindres = kwargs.pop("atellica_cyto_exclusion_resultat_cylindres", False)
        if atellica_cyto_exclusion_resultat_cylindres:
            self.liste_atellica_cyto_exclusion_resultat_cylindres = atellica_cyto_exclusion_resultat_cylindres.split(";")
        else:
            self.liste_atellica_cyto_exclusion_resultat_cylindres = []
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = Atellica800AstmSegmentH(type_segment="H", **kwargs)
        message.end = Atellica800AstmSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["priority", "action"],
            priority={"A": "R", "S": "R", worklist.ELSE: "R"},
            action={"del": "C", worklist.ELSE: "N"},
        ):
            seg_p = message.patients.add(
                patient_name="{} {}".format(
                    patient.last_name,
                    patient.first_name,
                )
            )
            for sample, priority, action in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        sample_id=sample.sample_id,
                        # Not diluted : 1
                        dilution_factor=order.dilution if order.dilution else "1",
                        priority=priority,
                        request_date=sample.request_date,
                        action_code=action,
                        specimen=sample.specimen if sample.specimen else "Urine",
                    )
        if message.patients:
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id
                )
            return worklist
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            lot = ""
            for seg_c in seg_p.commentaires:
                if seg_c.comment_type == "I" and seg_c.comment_text.startswith("Lot ID:"):
                    lot = seg_c.comment_text.split(":", 1)[1].strip()
                    break
            for seg_o in seg_p.orders:
                if seg_o.rack and seg_o.position:
                    rack = "{}-{}".format(seg_o.rack, seg_o.position)
                else:
                    rack = None
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        sample_id=lot or seg_o.sample_id,
                        control_lot=lot or seg_o.sample_id,
                        type="control"
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id,
                        rack=rack,
                        type="patient"
                    )
                cristaux = []
                cylindres = []
                for seg_r in seg_o.results:
                    flags = []
                    unit = seg_r.unit.strip()
                    if (
                        seg_r.code_test and
                        seg_r.code_test in self.atellica_cyto_cristaux.split(";") and
                        (not self.atellica_cyto_ignorer_resultats_na or seg_r.result != "N/A") and
                        not unit
                    ):
                        if "{}{}".format(seg_r.code_test, seg_r.result) in self.liste_atellica_cyto_exclusion_resultat_cristaux:
                            continue
                        cristaux.append("{}{}".format(seg_r.code_test, seg_r.result))
                    elif (
                        seg_r.code_test and
                        seg_r.code_test in self.atellica_cyto_cylindres.split(";") and
                        (not self.atellica_cyto_ignorer_resultats_na or seg_r.result != "N/A") and
                        not unit
                    ):
                        if "{}{}".format(seg_r.code_test, seg_r.result) in self.liste_atellica_cyto_exclusion_resultat_cylindres:
                            continue
                        cylindres.append("{}{}".format(seg_r.code_test, seg_r.result))
                    else:
                        if seg_r.abnormal_flags != "N":
                            flags.append(seg_r.abnormal_flags)
                        # Traitement des résultats qualitatifs
                        if unit:
                            code = seg_r.code_test
                        else:
                            code = "{}QUAL".format(
                                seg_r.code_test
                            )
                        sample.results.add(
                            code=code,
                            value=seg_r.result,
                            flags=flags,
                            unit=unit,
                            date=seg_o.result_date,
                            operator=seg_r.operator,
                            instrument=message.serial_number,
                        )
                if cristaux:
                    sample.results.add(
                        code="CRISTAUX",
                        value=";".join(cristaux),
                        instrument=message.serial_number,
                    )
                if cylindres:
                    sample.results.add(
                        code="CYLINDRES",
                        value=";".join(cylindres),
                        instrument=message.serial_number,
                    )
        return worklist
