# -*- coding: utf8 -*-
from __future__ import unicode_literals

import re

from driverng import Worklist

from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class M2000SegmentH(HeaderSegment):

    sender_id = HprimMultiple(
        5,
        name=HprimString(1, default="MCA"),
        software_version=HprimString(2),
        serial_number=HprimString(3),
        interface_version=HprimString(4),
    )

    receiver_id = HprimMultiple(
        10,
        host_name=HprimString(1),
        ip_adress=HprimString(2),
    )
    # P: Production
    # D: Debugging
    # Q: QC
    processing_id = HprimString(12, default="P")
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class M2000SegmentP(Segment):

    patient_id = HprimString(3)
    laboratory_patient_id = HprimString(4)
    instrument_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    mother_maiden_name = HprimString(7)
    birthday = HprimDate(8, fmt_date="%Y%m%d")

    # valeur admise M/F/U
    sex = HprimString(9)
    attending_physician = HprimString(14)
    location_code = HprimString((26, 1))
    location_name = HprimString((26, 2))
    orders = HprimForeign("O", "patient")


class M2000SegmentQ(Segment):

    starting_range_id = HprimMultiple(
        3,
        patient_id=HprimString(1),
        specimen_id=HprimString(2),
    )
    ending_range_id = HprimMultiple(
        4,
        patient_id=HprimString(1),
        specimen_id=HprimString(2),
    )
    instrument_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        name=HprimString(5),
    ))
    request_time_limit = HprimDate(6)
    begin_request_time = HprimDate(7)
    end_request_time = HprimDate(8)
    request_physician_name = HprimString(9)

    # Valeur possible pour Request Status Codes:
    #  A: Abort
    #  F: Final report
    #  N: New or edited results
    #  O: Orders and Demographics
    #  D: Demography only
    #  X: Request cannot be done
    status_code = HprimString(13)


class M2000SegmentO(Segment):

    specimen_id = HprimMultiple(
        3,
        specimen_id=HprimString(1),
        location_id=HprimString(2),
        position=HprimString(3),
    )
    instrument_specimen_id = HprimMultiple(
        4,
        specimen_id=HprimString(1),
        location_id=HprimString(2),
        position=HprimString(3),
    )

    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        name=HprimString(5),
        protocol=HprimString(6),
        test_qualifer=HprimString(7),
        result_qualifer=HprimString(8),
    ))

    # S : Stat
    # R : Routine
    priority = HprimString(6)
    request_date = HprimDate(7)
    collection_date = HprimDate(8)
    collection_id = HprimString(11)

    # C : Cancel
    # A : Add
    # N : Normal
    # Q : QC
    action_code = HprimString(12)
    specimen_type = HprimString((16, 1))
    specimen_source = HprimString((16, 2))
    order_physician = HprimString(17)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class M2000SegmentR(Segment):

    universal_test_id = HprimMultiple(
        3,
        code=HprimString(4),
        name=HprimString(5),
        protocol=HprimString(6),
        test_qualifer=HprimString(7),
        # F: Final
        # I: Interpreted
        # P: Preliminary
        # Favg: identifies the avevage of a set of final results
        # Pavg
        # Iavg
        result_type=HprimString(9),
    )
    data = HprimString(4)
    units = HprimString(5)
    referance_ranges = HprimMultiple(
        6,
        ref_range=HprimString(1),
        description=HprimString(2),
    )

    # valeur possible pour result Abnormal Flags:
    #  L: Less than normal range
    #  H: Higher than normal range
    #  LL: Less than extreme range
    #  HH: Higher than extreme range
    #  QC: Result based on a QC out of range
    #  >: Above dynamic range of assay
    #  <: Below dynamic range of assay
    #  EXP: Result based on expired reagent
    #  *: Additional abnormal flags may be defined as needed by instruments
    abnormal_flag = HprimString(7)
    nature_abnormality = HprimString(8)

    # Valeur possible pour Result Status:
    #  F: Final results
    #  R: Repeat
    #  X: Test cannot be completed
    result_status = HprimString(9)
    operator_id_name = HprimString((11, 1))
    operator_id_approver = HprimString((11, 2))
    date_started = HprimDate(12)

    # date du resultat
    date_completed = HprimDate(13)
    instrument_id = HprimString(14)


class M2000SegmentC(Segment):

    # I: Instrument
    # L: Lis
    # D: Data Management Systems
    comment_source = HprimString(3)
    comment_text = HprimString(4)

    # G: Generic free form comments entered by the lab operator
    # I: Instrument generated exception string
    comment_type = HprimString(5)


class M2000SegmentM(Segment):

    abbott_record_type = HprimMultiple(
        3,
        record_class=HprimString(1),
        instrument_record_type=HprimString(2),
    )


class M2000SegmentL(Segment):
    # N : Normal termination
    termination_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "m2000",
        "label": "M2000",
    }
    SEGMENT_CLASS_MAP = {
        "H": M2000SegmentH,
        "P": M2000SegmentP,
        "O": M2000SegmentO,
        "Q": M2000SegmentQ,
        "R": M2000SegmentR,
        "C": M2000SegmentC,
        "M": M2000SegmentM,
        "L": M2000SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = M2000SegmentH(type_segment="H", **kwargs)
        message.end = M2000SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
        ):
            physician = [p for p in patient.physicians if p.type.lower() == 'prescriber']
            seg_p = msg.patients.add(
                laboratory_patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                mother_maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                attending_physician=physician[0].code if physician else "",
                location_name=patient.location,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    priority=priority,
                    request_date=sample.request_date,
                    collection_date=sample.collection_date,
                    code_action=action,
                    report_type="Q" if context["query"] else "O",
                )
                seg_o.specimen_id.specimen_id = sample.sample_id
                for order in sample.orders:
                    t = seg_o.universal_test_id.add()
                    t.code = order.code

        if msg.patients:
            return msg
        elif context["query"]:
            msg.end.code = "N"
            for seg_q in context["query"].queries:
                seg_query = msg.queries.add()
                seg_query.starting_range_id.specimen_id = seg_q.starting_range_id.specimen_id
                seg_query.instrument_test_id.code = "ALL"
                seg_query.status_code = "X"
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    sample_id=seg_q.starting_range_id.specimen_id.strip(),
                    type='patient',
                )
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                is_qc = seg_o.action_code == "Q"
                if is_qc:
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=seg_o.specimen_id.specimen_id.strip(),
                        control_lot=seg_o.specimen_id.specimen_id.strip(),
                    )
                else:
                    if (
                            seg_o.instrument_specimen_id.location_id and
                            seg_o.instrument_specimen_id.position
                    ):
                        rack = "{}-{}".format(
                            seg_o.instrument_specimen_id.location_id,
                            seg_o.instrument_specimen_id.position,
                        )
                    else:
                        rack = None
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.specimen_id.specimen_id.strip(),
                        rack=rack,
                    )
                if seg_o.report_type == "X":
                    for test in seg_o.universal_test_id:
                        comments = []
                        for seg_c in seg_o.commentaires:
                            comments.append(seg_c.comment_text.split(":")[0].strip())
                        if comments:
                            sample.results.add(
                                code=test.code,
                                value="ERREUR",
                                flags=comments,
                            )
                for seg_r in seg_o.results:
                    if seg_r.abnormal_flag:
                        alarmes = [seg_r.abnormal_flag]
                    else:
                        alarmes = []

                    commentaires = []
                    for seg_c in seg_r.commentaires:
                        if seg_c.comment_type:
                            commentaires.append(seg_c.comment_text)

                    # Le M2000 transmet pour le paramètre HPV plusieurs
                    # résultats dans le champ R4 séparé avec le séparateur ;
                    # Hors protocole!!!
                    # Ce champ inclus une partie du code test et le résultat...
                    # format du champ: <test><space>(<valeur>)
                    # Vu avec Abbott, ils ne peuvent corriger cette anomalie.
                    resultats = []
                    if all([c in seg_r.data for c in "()"]):
                        for e in seg_r.data.split(";"):
                            test, resu = re.match(r"(.*)\((.*)\)", e).groups()
                            resultats.append({
                                'code': "{}{}_{}".format(
                                    seg_r.universal_test_id.code,
                                    seg_r.universal_test_id.result_type,
                                    test.strip().replace(" ", "_"),
                                ),
                                'value': resu,
                            })
                    else:
                        resultats = [{
                            'code': "{}{}".format(
                                seg_r.universal_test_id.code,
                                seg_r.universal_test_id.result_type,
                            ),
                            'value': seg_r.data,
                        }]

                    for code_and_value in resultats:
                        sample.results.add(
                            date=seg_r.date_completed,
                            flags=alarmes,
                            comments=commentaires,
                            operator=seg_r.operator_id_name,
                            **code_and_value
                        )
        return worklist
